/* Variable validation command file for IBM DB2 for OS/2 - Server */

keyword_name = TRANSLATE(arg(1))
PDFRSLT.0 = 0
result = 0
value = PDFVAR.5
error_message = 'Invalid value for'
unknown_message = 'Unknown keyword:'

parse source . . me
lastslash = lastpos('\',me)
mydir = substr(me, 1, lastslash)

call RxFuncAdd 'DB2IValidateKeywordValue', mydir'DB2RXVAL.dll', 'DB2IValidateKeywordValue'

rc = DB2IValidateKeywordValue(keyword_name, value);

select
    when rc = 'VALID' then
        result = 0
    when rc = 'INVALID' then
    do
        PDFRSLT.0 = 1
        PDFRSLT.1 = error_message keyword_name
        result = 1
    end
    when rc = 'UNKNOWN KEYWORD' then
    do
        PDFRSLT.0 = 1
        PDFRSLT.1 = unknown_message keyword_name
        result = 1
    end

    otherwise
    do
        PDFRSLT.0 = 1
        PDFRSLT.1 = 'Unknown error:' keyword_name
        result = 1
    end
end

if (result = 0) & (arg() > 1) then
    select
        when keyword_name = 'MAXDARI' then
        do
        /* MAXDARI keyword validation */
            max_coordagents = ARG(2)
            if \(value = -1 | (value >= 1 & value <= max_coordagents)) then
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        when keyword_name = 'MAX_COORDAGENTS' THEN
        do
        /* MAX_COORDAGENTS keyword validation */
            maxagents = ARG(2)
            if \(value = -1 | (value >= 1 & value <= maxagents)) then
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        when keyword_name = 'NUM_INITAGENTS' then
        do
        /* NUM_INITAGENTS keyword validation */
            if \(value >= 1 & value <= ARG(2)) then
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        WHEN keyword_name = 'NUM_POOLAGENTS' THEN
        do
        /* NUM_POOLAGENTS keyword validation */
            IF \(value = -1 | (value >= 1 & value <= ARG(2))) THEN
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        WHEN keyword_name = 'DB2NBINTRLISTENS' THEN
        do
        /* DB2NBINTRLISTENS keyword validation */
            CALL validate_adapter_values ARG(2), value, keyword_name
            IF rc = 0 THEN
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        WHEN keyword_name = 'DB2NBSESSIONS' THEN
        do
        /* DB2NBSESSIONS keyword validation */
            CALL validate_adapter_values ARG(2), value, keyword_name
            IF rc = 0 THEN
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        WHEN keyword_name = 'DB2NBXTRANCBS' THEN
        do
        /* DB2NBXTRANCBS keyword validation */
            CALL validate_adapter_values ARG(2), value, keyword_name
            IF rc = 0 THEN
            do
                PDFRSLT.0 = 1
                PDFRSLT.1 = error_message keyword_name
                result = 1
            end
        end

        OTHERWISE
        NOP
    end

RETURN result


/* Subroutine to validate string keywords value.                             */
/* Takes two argument, the keyword to be validated and the length,           */
/* and returns rc = 1 if it is valid, 0 if it is not.                        */
validate_char:
  rc = 0
  name = ARG(1)
  chkleng = ARG(2)
  length = LENGTH(name)
  invalidchar = ''
  IF length = 0 THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  DO
    IF length > chkleng THEN
      RETURN
    ELSE
    DO
      first = LEFT(name, 1)
      DO position = 1 TO length
        IF POS(first, invalidchar) > 0 THEN
          RETURN
        first = LEFT(RIGHT(name, (length - position)), 1)
      END
    END
    rc = 1
  END
RETURN



/* Subroutine to validate hex number value.                                  */
/* Takes one argument,                                                       */
/* and returns rc = 1 if it is valid, 0 if it is not.                        */
validate_hexnum:
  rc = 0
  name = TRANSLATE(ARG(1))
  expectlen = arg(2)
  length = LENGTH(name)
  IF length > expectlen THEN
    RETURN
  ELSE
  DO
    DO position = 1 TO (length - 1)
    IF POS(first, '0123456789ABCDEF') < 0 THEN
       RETURN
      first = LEFT(RIGHT(name, (length - position)), 1)
    END
    rc = 1
  END
RETURN



/* Subroutine to validate authority group name keywords                      */
/* Takes one argument, the name of a system authority group, and returns     */
/* rc = 1 if it is valid, 0 if it is not.                                    */
validate_group:
  rc = 0
  name = ARG(1)
  length = LENGTH(name)
  IF length = 0 THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  DO
    IF length > 8 THEN
      RETURN
    ELSE
    DO
      uppername = TRANSLATE(name)
      IF uppername = 'ADMINS' | uppername = 'GUESTS' | uppername = 'LOCAL' | uppername = 'PUBLIC' | uppername = 'USERS' THEN
        RETURN
      ELSE
      DO
        IF length >= 3 THEN
        DO
          first3letters = LEFT(uppername, 3)
          IF first3letters = 'IBM' | first3letters = 'SQL' | first3letters = 'SYS' THEN
            RETURN
          ELSE
          DO
            first = LEFT(name, 1)
            last = RIGHT(name, 1)
            IF DATATYPE(first, N) | last = '$' THEN
              RETURN
            ELSE
            DO
              DO position = 1 TO (length - 1)
                IF \(DATATYPE(first, A) | first = '@' | first = '#' | first = '$' | first = '_') THEN
                  RETURN
                first = LEFT(RIGHT(name, (length - position)), 1)
              END
              rc = 1
            END
          END
        END
      END
    END
  END
RETURN


/* Subroutine to validate FILESERVER and OBJECTNAME keywords.                */
/* Takes one argument, the keyword to be validated, and returns rc = 1 if it */
/* is valid, 0 if it is not.                                                 */
validate_object:
  rc = 0
  name = ARG(1)
  length = LENGTH(name)
  IF length = 0 | name = '*' THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  DO
    IF length > 48 THEN
      RETURN
    ELSE
    DO
      first = LEFT(name, 1)
      DO position = 1 TO length
        IF first = ' ' | first = '/' | first = '\' | first = ':' | first = ';' | first = ',' | first = '*' | first = '?' THEN
          RETURN
        first = LEFT(RIGHT(name, (length - position)), 1)
      END
    END
    rc = 1
  END
RETURN


/* Subroutine to validate database names.                                    */
/* Takes one argument, the name to be validated, and returns rc = 1 if it    */
/* is valid, 0 if it is not.                                                 */
validate_dbname:
  rc = 0
  name = ARG(1)
  length = LENGTH(name)
  IF length = 0 THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  DO
    IF length > 8 THEN
      RETURN
    ELSE
    DO
      first = LEFT(name, 1)
      last = RIGHT(name, 1)
      IF \(DATATYPE(first, M) | first = '@' | first = '#' | first = '$') THEN
        RETURN
      ELSE
      DO
        DO position = 1 TO length
          IF \(DATATYPE(first, A) | first = '@' | first = '#' | first = '$' | first = '_') THEN
            RETURN
          first = LEFT(RIGHT(name, (length - position)), 1)
        END
        rc = 1
      END
    END
  END
RETURN


/* Subroutine to validate DB2NBADAPTERS keyword.                             */
/* Takes one argument, comma-delimited string of adapter numbers, and        */
/* returns rc = 1 if it is valid, 0 if it is not.                            */
validate_adapters:
  rc = 0
  string = ARG(1)
  IF string = '' THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  number_of_adapters = 0
  DO i = 0 TO 15
    adapter_numbers.i = 0
  END
  comma_pos = POS(',', string)
  DO WHILE \(comma_pos = 0)
    PARSE VAR string number','string
    number = STRIP(number)
    IF number = '' | VERIFY(number, '1234567890') THEN
      RETURN
    ELSE
    DO
      IF number > 15 THEN
        RETURN
      ELSE
      DO
        IF (adapter_numbers.number = 1) THEN
          RETURN
        ELSE
        DO
          adapter_numbers.number = 1
          number_of_adapters = number_of_adapters + 1
          IF number_of_adapters > 16 THEN
            RETURN
        END
      END
    END
    comma_pos = POS(',', string)
  END

  IF string = '' THEN
    RETURN
  ELSE
  DO
    number = string
    number = STRIP(number)
    IF VERIFY(number, '1234567890') THEN
      RETURN
    ELSE
    DO
      IF number > 15 THEN
        RETURN
      ELSE
      DO
        IF adapter_numbers.number THEN
          RETURN
        ELSE
        DO
          adapter_numbers.number = 1
          number_of_adapters = number_of_adapters + 1
          IF number_of_adapters > 16 THEN
            RETURN
        END
      END
    END
  END
  rc = 1
RETURN


/* Subroutine to validate DB2NBINTRLISTENS or DB2NBSESSIONS keywords.        */
/* Takes three arguments, comma-delimited strings of adapter and value       */
/* numbers, and a name which indicates which keyword to check.               */
/* Returns rc = 1 if it is valid, 0 if it is not.                            */
validate_adapter_values:
  rc = 0
  string = ARG(2)
  keyword = ARG(3)
  IF keyword = 'DB2NBINTRLISTENS' THEN
  DO
    minvalue = 0
    maxvalue = 10
  END
  ELSE
  DO
    IF keyword = 'DB2NBSESSIONS' | keyword = 'DB2NBXTRANCBS' THEN
    DO
      minvalue = 5
      maxvalue = 254
    END
    ELSE
      RETURN
  END

  IF string = '' THEN
  DO
    rc = 1
    RETURN
  END
  ELSE
  DO
    number_of_adapters = WORDS(TRANSLATE(ARG(1), ' ', ','))
    number_of_values = 0
    comma_pos = POS(',', string)
    DO WHILE \(comma_pos = 0)
      PARSE VAR string number','string
      number = STRIP(number)
      IF \(number = '') THEN
      DO
        IF VERIFY(number, '1234567890') THEN
          RETURN
        ELSE
        DO
          IF number < minvalue | number > maxvalue THEN
            RETURN
          ELSE
          DO
            number_of_values = number_of_values + 1
            IF number_of_values > 16 THEN
              RETURN
          END
        END
      END
      ELSE
      DO
        number_of_values = number_of_values + 1
        IF number_of_values > 16 THEN
          RETURN
      END
      comma_pos = POS(',', string)
    END
  END
  IF \(number = '') THEN
  DO
    IF VERIFY(number, '1234567890') THEN
      RETURN
    ELSE
    DO
      IF number < minvalue | number > maxvalue THEN
        RETURN
      ELSE
      DO
        number_of_values = number_of_values + 1
        IF number_of_values > 16 THEN
          RETURN
      END
    END
  END
  ELSE
  DO
    number_of_values = number_of_values + 1
    IF number_of_values > 16 THEN
      RETURN
  END
  IF \(number_of_values = number_of_adapters) THEN
    RETURN
  rc = 1
RETURN

